<?php
if (!class_exists('Cryptojs_MARC4')) :
/**
 * Partial Crypto-JS v1.1.0 conversion to PHP
 * Works with PHP 4 (>= 4.3) and 5
 * Methods should be called statically (e.g. Cryptojs::MD5('message');)
 *   Implemented: MD5, SHA-1, SHA-256, AES, MARC4, HMAC-MD5, HMAC-SHA1, HMMAC-SHA256, PBKDF2, Utilities
 *   Not Implemented: Rabbit
 *   Note: Rabbit appears to use integers that are too large for PHP (32-bit) to natively handle
 *   Note: SHA-256 and HMAC-SHA256 require 'hash' or 'mhash' (also uses integers that are too large)
 *   Note: CBC and OFB have not been implemented as usable methods, but they can be specified when calling the AES methods
 * 
 * Original JavaScript version:
 *   Crypto-JS v1.1.0
 *   http://code.google.com/p/crypto-js/
 *   Copyright (c) 2009, Jeff Mott. All rights reserved.
 *   http://code.google.com/p/crypto-js/wiki/License (New BSD License)
 */
class Cryptojs_MARC4 {

	function encrypt($message, $password) {
		// Convert to bytes
		$m = Cryptojs_util::stringToBytes($message);
		
		// Generate random IV
		$iv = Cryptojs_util::randomBytes(16);
		
		// Generate key
		$k = Cryptojs::PBKDF2($password, Cryptojs_util::bytesToString($iv), 32, array('asBytes' => true));
		
		// Encrypt
		Cryptojs_MARC4::_marc4(&$m, $k, 1536);
		
		$temp_array = $iv;
		foreach($m as $m_val) {
			$temp_array[] = $m_val;
		}
		
		// Return ciphertext
		return Cryptojs_util::bytesToBase64($temp_array);
	}
	
	function decrypt($ciphertext, $password) {
		// Convert to bytes
		$c = Cryptojs_util::base64ToBytes($ciphertext);
		
		// Separate IV and message
		$iv = array_splice(&$c, 0, 16);
		
		// Generate key
		$k = Cryptojs::PBKDF2($password, Cryptojs_util::bytesToString($iv), 32, array('asBytes' => true));
		
		// Decrypt
		Cryptojs_MARC4::_marc4(&$c, $k, 1536);
		
		// Return plaintext
		return Cryptojs_util::bytesToString($c);
	}
	
	function _marc4(&$m, $k, $drop) {
		// State variables
		$i; $j; $s; $temp;
		
		// Key Setup
		for ($i = 0, $s = array(); $i < 256; $i++) $s[$i] = $i;
		for ($i = 0, $j = 0; $i < 256; $i++) {
			$j = ($j + $s[$i] + $k[$i % count($k)]) % 256;
			
			// Swap
			$temp = $s[$i];
			$s[$i] = $s[$j];
			$s[$j] = $temp;
		}
		
		// Clear counters
		$i = $j = 0;
		
		// Encryption
		for ($k = 0 - $drop; $k < count($m); $k++) {
			$i = ($i + 1) % 256;
			$j = ($j + $s[$i]) % 256;
			
			// Swap
			$temp = $s[$i];
			$s[$i] = $s[$j];
			$s[$j] = $temp;
			
			// Stop here if we're still dropping keystream
			if ($k < 0) continue;
			
			// Encrypt
			$m[$k] ^= $s[($s[$i] + $s[$j]) % 256];
		}
	}
}
endif;
?>