<?php defined('ABSPATH') OR die();

$plugin_url = plugins_url( plugin_basename(dirname(dirname(__FILE__))) );

$sub = $_GET['sub'];
if ($sub == 'misc')
	$class_misc = 'class="current"';
else if ($sub == 'integration')
	$class_integration = 'class="current"';
else if ($sub == 'uninstall')
	$class_uninstall = 'class="current"';
else {
	$class_keypair = 'class="current"';
	$sub = 'keypair';
}

$blog_charset = get_option('blog_charset');
$blog_charset = trim(strtoupper($blog_charset));
if ($blog_charset != 'UTF-8') {
	print '<div class="error"><p><strong>' . sprintf(__('This plugin is tested to work with UTF-8. Your <a href="%s">blog settings</a> are currently using the following character encoding: %s', $this->text_domain), 'options-reading.php', $blog_charset) . '</strong></p></div>';
}

if (!extension_loaded('openssl')) {
	print '<div class="error"><p><strong>' . __('OpenSSL doesn\'t appear to be installed. This plugin relies on OpenSSL and won\'t work until it\'s been installed.', $this->text_domain) . '</strong></p></div>';
}
else if (!function_exists('openssl_private_decrypt')) {
	print '<div class="error"><p><strong>' . __('The <code>openssl_private_decrypt</code> function appears to be disabled. This plugin won\'t work until it\'s been enabled. Check the <code>disable_functions</code> setting in your php.ini file.', $this->text_domain) . '</strong></p></div>';
}

// WP forces GPC to be escaped, but nothing here should fall into that scenario
// generate and save a new keypair
if (isset($_POST['rsa_numbits']) && !empty($_POST['rsa_numbits']) && is_numeric($_POST['rsa_numbits'])) {
	if (current_user_can('manage_options')) {
		check_admin_referer('generate_semisecure-login-reimagined');
		if ($_POST['force_alt_method'] == 'yes')
		$this->force_alt_keypair_generation_method = TRUE;
		if ($_POST['show_debug_info'] == 'yes')
			$show_debug_info = true;
		$rsa_numbits = $_POST['rsa_numbits'];
		$rsa_publicexponent = is_numeric($_POST['rsa_publicexponent']) ? $_POST['rsa_publicexponent'] : 10001;
		$successful = $this->store_new_rsa_keypair($rsa_numbits, $rsa_publicexponent);
		if ($successful) {
			print '<div id="message" class="updated fade"><p><strong>' . __('The new keypair was successfully generated and saved.', $this->text_domain) . '</strong>';
		}
		else {
			print '<div id="message" class="error"><p><strong>' . __('Keypair could not be created!', $this->text_domain) . '</strong>';
		}
		if ($show_debug_info) {
			print '<ul>';
			foreach ($this->debug_info as $val) {
				print '<li>' . $val . '</li>';
			}
			print '</ul>';
		}
		print '</p></div>';
		}
		else {
			print '<div id="message" class="error"><p><strong>' . __('Ooops... looks like you don\'t have the correct privileges to make this change.', $this->text_domain) . '</strong></p></div>';
		}
}
// miscellaneous settings
else if (isset($_POST['where_to_encrypt']) && !empty($_POST['where_to_encrypt']) && $_POST['where_to_encrypt'] == 'postback') {
	if (current_user_can('manage_options')) {
		check_admin_referer('misc_semisecure-login-reimagined');
		update_option('semisecurelogin_reimagined_encrypt_admin', isset($_POST['do_admin_encrypt']));
		update_option('semisecurelogin_reimagined_encrypt_login', isset($_POST['do_login_encrypt']));
		$secret_key_algo = $_POST['secret_key_algo'];
		if (in_array($secret_key_algo, array('marc4', 'aes-cbc', 'aes-ofb')))
			update_option('semisecurelogin_reimagined_secretkey_algo', $secret_key_algo);
		print '<div id="message" class="updated fade"><p><strong>' . __('The settings were successfully updated.', $this->text_domain) . '</strong></p></div>';
	}
	else {
		print '<div id="message" class="error"><p><strong>' . __('Ooops... looks like you don\'t have the correct privileges to make this change.', $this->text_domain) . '</strong></p></div>';
	}
}
// uninstall
else if (isset($_POST['uninstall_semisecure_login_reimagined']) && !empty($_POST['uninstall_semisecure_login_reimagined']) && $_POST['uninstall_semisecure_login_reimagined'] == 'yes') {
	if (current_user_can('manage_options')) {
		check_admin_referer('uninstall_semisecure-login-reimagined');
		delete_option('semisecurelogin_reimagined_encrypt_admin');
		delete_option('semisecurelogin_reimagined_encrypt_login');
		delete_option('semisecurelogin_reimagined_rsa_keys');
		delete_option('semisecurelogin_reimagined_secretkey_algo');
		$deactivate_url = 'plugins.php?action=deactivate&amp;plugin=semisecure-login-reimagined%2Fplugin.php';
		if(function_exists('wp_nonce_url')) { 
			$deactivate_url = wp_nonce_url($deactivate_url, 'deactivate-plugin_semisecure-login-reimagined/plugin.php');
		}
		print '<div id="message" class="updated fade"><p><strong>' . sprintf(__('<a href="%s">Click here</a> to finish the uninstallation and this plugin will be deactivated automatically.', $this->text_domain), $deactivate_url) . '</strong></p></div>';
	}
	else {
		print '<div id="message" class="error"><p><strong>' . __('Ooops... looks like you don\'t have the correct privileges to make this change.', $this->text_domain) . '</strong></p></div>';
	}
}

$do_encrypt_admin = get_option('semisecurelogin_reimagined_encrypt_admin');
$do_encrypt_login = get_option('semisecurelogin_reimagined_encrypt_login');
$secret_key_algo = get_option('semisecurelogin_reimagined_secretkey_algo');

$keys_array = get_option('semisecurelogin_reimagined_rsa_keys');
if ($keys_array === FALSE OR !is_array($keys_array)) {
	// attempt to fix a line-ending/count issue if the keypair was generated via the JavaScript method
	if ($this->fix_rsa_keypair_serialization()) {
		$keys_array = get_option('semisecurelogin_reimagined_rsa_keys');
		print '<div id="message" class="updated fade"><p><strong>' . __('The keypair was in a bad state. This has been automatically fixed.') . '</strong></p></div>';
	}
}
// make sure the keypair newlines are set as PHP_EOL
if ($this->consolidate_rsa_keypair_newlines()) {
	$keys_array = get_option('semisecurelogin_reimagined_rsa_keys');
	// don't print out a message in this case
}
?>
<div class="wrap">
<div class="icon32" id="icon-options-general"><br/></div>
<h2><?php _e('Semisecure Login Reimagined', $this->text_domain); ?></h2>
<p>
	<?php _e('This plugin uses a combination of public-key (RSA) and secret-key (MARC4 or AES) encryption to encrypt your password on the client-side before transmission. A nonce is used to help prevent replay attacks. This provides a &quot;semisecure&quot; login environment. For full security you should use an SSL certificate.', $this->text_domain); ?>
</p>
<ul class="subsubsub">
  <li><a <?php echo $class_keypair; ?> href="?page=semisecureloginreimagined"><?php _e('Keypair Settings', $this->text_domain); ?></a> | </li>
  <li><a <?php echo $class_misc; ?> href="?page=semisecureloginreimagined&amp;sub=misc"><?php _e('Misc Settings', $this->text_domain); ?></a> | </li>
  <li><a <?php echo $class_integration; ?> href="?page=semisecureloginreimagined&amp;sub=integration"><?php _e('Integration', $this->text_domain); ?></a> | </li>
  <li><a <?php echo $class_uninstall; ?> href="?page=semisecureloginreimagined&amp;sub=uninstall"><?php _e('Uninstall', $this->text_domain); ?></a></li>
</ul>
<div style="clear:both;"></div>

<?php if ($sub == 'keypair') : ?>

<p>
<?php
// check if a keypair already exists
if (SemisecureLoginReimagined::is_rsa_key_ok()) {
?>
	<?php printf(__('The current key is <strong>%d</strong> bits in length, and the public-exponent is <strong>%s</strong>', $this->text_domain), $keys_array['numbits'], '0x' . $keys_array['publicexponent']); ?>
	<table border="0">
		<tr valign="top">
			<td><?php _e('Modulus:', $this->text_domain); ?></td>
			<td><pre><?php echo wordwrap($keys_array['modulus'], 64, "\n", true) ?></pre></td>
		</tr>
	</table>
<?php
} else {
?>
	<span style="color:red;"><strong><?php _e('WARNING:', $this->text_domain); ?></strong><br /><?php _e('No current keypair found! Please remedy the situation by generating a new keypair below.', $this->text_domain); ?></span>
<?php
}
?>
</p>
<p>
	<?php _e('<em>&quot;RSA claims that 1024-bit keys are likely to become crackable some time between 2006 and 2010 and that 2048-bit keys are sufficient until 2030. An RSA key length of 3072 bits should be used if security is required beyond 2030.&quot;</em> -<a href="http://en.wikipedia.org/wiki/Key_size#Asymmetric_algorithm_key_lengths" target="_blank">Wikipedia</a>', $this->text_domain); ?>
</p>
<h3><?php _e('Generate New Keypair', $this->text_domain); ?></h3>
<p>
	<?php _e('Use the following options to generate &amp; store a new RSA keypair.', $this->text_domain); ?>
</p>
<p>
	<?php _e('<em>A lower number of bits requires less processing power, but also limits the amount of data it can encode. More bits corresponds to both better encryption and higher processing load.</em>', $this->text_domain); ?>
</p>
<form method="post" action="">
	<table class="form-table">
		<tr valign="top">
			<th scope="row"><?php _e('Number of bits', $this->text_domain); ?></th>
			<td>
				<select name="rsa_numbits">
					<option value="512"><?php _e('512 bits', $this->text_domain); ?></option>
					<option value="1024" selected="selected"><?php _e('1024 bits', $this->text_domain); ?></option>
					<option value="2048"><?php _e('2048 bits', $this->text_domain); ?></option>
					<option value="3072"><?php _e('3072 bits', $this->text_domain); ?></option>
				</select>
				<br/><?php _e('1024 bits is currently recommended based on the tradeoff between performance and security.', $this->text_domain); ?>
			</td>
		</tr>
		<tr valign="top">
			<th scope="row"><?php _e('Public-exponent', $this->text_domain); ?></th>
			<td>
				<select name="rsa_publicexponent">
					<option value="3"><?php _e('3 (0x3)', $this->text_domain); ?></option>
					<option value="10001" selected="selected"><?php _e('F4 (0x10001)', $this->text_domain); ?></option>
				</select>
				<br /><?php _e('OpenSSL defaults to F4; 0x3 has slightly faster encryption times<br /><em>Note: the alternative keypair generation method will not respect this value</em>', $this->text_domain); ?>
			</td>
		</tr>
		<tr valign="top">
			<td colspan="2">
				<hr style="text-align:center;width:50%;" />
			</td>
		</tr>
		<tr valign="top">
			<th scope="row"><?php _e('Force alt method', $this->text_domain); ?></th>
			<td>
				<label><input type="checkbox" name="force_alt_method" value="yes" />  <?php _e('Yes', $this->text_domain); ?> </label>
				<br /><?php _e('There are two keypair generation methods. The default method makes a direct call against openssl on the server. The alternative method uses PHP\'s built-in openssl functions. The alternative method will automatically be used if: PHP\'s execution functions are disabled, safe mode is enabled, or openssl is not in the location specified in the advanced options file. You can also force the alternative method using this option.', $this->text_domain); ?>
			</td>
		</tr>
		<tr valign="top">
			<th scope="row"><?php _e('Show debug info', $this->text_domain); ?></th>
			<td>
				<label><input type="checkbox" name="show_debug_info" value="yes" /> <?php _e('Yes', $this->text_domain); ?> </label>
				<br /><?php _e('If the keypair is not generating correctly, then you can use this option to find out why.', $this->text_domain); ?>
			</td>
		</tr>
	</table>
	<p class="submit">
		<?php wp_nonce_field('generate_semisecure-login-reimagined'); ?>
		<input type="submit" name="Submit" value="<?php _e('Generate Key &raquo;', $this->text_domain); ?>" />
	</p>
</form>
<p>
	<?php printf(__('Click <a href="%s" target="_blank">here</a> to create a new RSA keypair via JavaScript in your browser.', $this->text_domain), $plugin_url.'/help/generate_key.php'); ?>
</p>

<?php elseif ($sub == 'misc') : ?>

<form method="post" action="">
	<table class="form-table">
		<tr valign="top">
			<th scope="row"><?php _e('User login', $this->text_domain); ?></th>
			<td>
				<label><input type="checkbox" name="do_login_encrypt" value="yes" <?php if ($do_encrypt_login) { echo 'checked="checked"'; } ?> /> <?php _e('Encrypt passwords when logging in?', $this->text_domain); ?> </label>
				<br/><?php _e('This includes the wp-login.php page as well as any plugins that implement the login_head and login_form hooks.', $this->text_domain); ?>
			</td>
		</tr>
		<tr valign="top">
			<th scope="row"><?php _e('User administration', $this->text_domain); ?></th>
			<td>
				<label><input type="checkbox" name="do_admin_encrypt" value="yes" <?php if ($do_encrypt_admin) { echo 'checked="checked"'; } ?> /> <?php _e('Encrypt passwords when managing users?', $this->text_domain); ?> </label>
				<br/><?php _e('This includes changing or setting a password when: editing your own user (wp-admin/profile.php), editing another user (wp-admin/user-edit.php), and creating a new user (wp-admin/user-new.php).', $this->text_domain); ?>
			</td>
		</tr>
		<tr valign="top">
			<th scope="row"><?php _e('Secret-key algorithm', $this->text_domain); ?></th>
			<td>
				<label><input type="radio" name="secret_key_algo" value="marc4" <?php if ($secret_key_algo == 'marc4') echo 'checked="checked"'; ?> /> <?php _e('MARC4', $this->text_domain); ?> </label>
				<label><input type="radio" name="secret_key_algo" value="aes-cbc" <?php if ($secret_key_algo == 'aes-cbc') echo 'checked="checked"'; ?> /> <?php _e('AES (CBC)', $this->text_domain); ?> </label>
				<label><input type="radio" name="secret_key_algo" value="aes-ofb" <?php if ($secret_key_algo == 'aes-ofb') echo 'checked="checked"'; ?> /> <?php _e('AES (OFB)', $this->text_domain); ?> </label>
				<br/><?php _e('MARC4 (Modified Allegedly RC4) is based on RC4, a widely-used stream cipher. This modified version corrects certain weaknesses found in RC4. The Advanced Encryption Standard (AES) is a block cipher, and an encryption standard adopted by the U.S. government. CBC (cipher-block chaining) is one of the most common block cipher modes. OFB (output feedback) is a mode that turns a block cipher into a synchronous stream cipher.', $this->text_domain); ?>
			</td>
		</tr>
	</table>
	<p class="submit">
		<input type="hidden" name="where_to_encrypt" value="postback" />
		<?php wp_nonce_field('misc_semisecure-login-reimagined'); ?>
		<input type="submit" name="Submit" value="<?php _e('Update Options &raquo;', $this->text_domain); ?>" />
	</p>
</form>

<?php elseif ($sub == 'integration') : ?>

<p>
  <?php _e('By default, this plugin will encrypt your password on the <code>wp-login.php</code> page. Optionally, it can also encrypt your password on the <code>wp-admin/profile.php</code>, <code>wp-admin/user-edit.php</code>, and <code>wp-admin/user-new.php</code> pages. Here\'s a quick guide showing how you can integrate with this plugin to encrypt passwords on other pages too (most likely via a plugin).', $this->text_domain); ?>
</p>
<p>
  <?php _e('<em>Note: v3 of this plugin has made some (breaking) changes from the v2 integration</em>', $this->text_domain); ?>
</p>
<p>
  <?php _e('For starters, let\'s assume you have a form like the following (this only works for forms where <code>method="post"</code>):', $this->text_domain); ?>
</p>
<pre style="border:1px solid #000;margin:15px;overflow:auto;padding:5px;">
&lt;form id="loginform" method="post" action=""&gt;
	&lt;input type="password" id="user_pwd" name="pwd" /&gt;
	&lt;!-- <?php _e('Other form elements', $this->text_domain); ?> --&gt;
&lt;/form&gt;
</pre>
<p>
  <?php _e('The basics of encrypting are as follows:', $this->text_domain); ?>
</p>
<pre style="border:1px solid #000;margin:15px;overflow:auto;padding:5px;">
&lt;php
  // <?php echo __('Make sure that all the external JavaScript is available', $this->text_domain) . "\n"; ?>
  // <?php echo __('Optionally, pass TRUE into this method if the page doesn\'t automatically call wp_print_scripts', $this->text_domain) . "\n"; ?>
  SemisecureLoginReimagined::enqueue_js();
?&gt;
&lt;script type="text/javascript"&gt;
jQuery(document).ready(function($) {
  // <?php echo __('Bind to the form\'s submit event', $this->text_domain) . "\n"; ?>
  $('form#loginform').submit(function() {
    // <?php echo __('Collect the password(s) and form name(s)', $this->text_domain) . "\n"; ?>
    var password = $('#user_pwd').val();
    var name = 'pwd';

    // <?php echo __('Pass the needed PHP values over to the JavaScript side', $this->text_domain) . "\n"; ?>
    var public_n = '&lt;?php echo SemisecureLoginReimagined::public_n(); ?&gt;';
    var public_e = '&lt;?php echo SemisecureLoginReimagined::public_e(); ?&gt;';
    var uuid = '&lt;?php echo SemisecureLoginReimagined::uuid(); ?&gt;';
    var nonce_js = '&lt;?php echo SemisecureLoginReimagined::nonce_js(); ?&gt;';
    var max_rand_chars = '&lt;?php echo SemisecureLoginReimagined::max_rand_chars(); ?&gt;';
    var rand_chars = '&lt;?php echo addslashes(SemisecureLoginReimagined::rand_chars()); ?&gt;';
    var secret_key_algo = '&lt;?php echo SemisecureLoginReimagined::secret_key_algo(); ?&gt;';
    
    // <?php echo __('Encrypt the password(s)', $this->text_domain) . "\n"; ?>
    // <?php echo __('This function will return an Array on success or FALSE on failure', $this->text_domain) . "\n"; ?>
    var arr = SemisecureLoginReimagined.encrypt(password, name, nonce_js, public_n, public_e, uuid, secret_key_algo, rand_chars, max_rand_chars);
    
    if (arr) {
      // <?php echo __('Loop through the array and append the controls to the form', $this->text_domain) . "\n"; ?>
      for (var i = 0; i &lt; arr.length; i++) {
        $('form#loginform').append(arr[i]);
      }
      
      // <?php echo __('Finally, don\'t submit the plain-text password(s)', $this->text_domain) . "\n"; ?>
      // <?php echo __('One option is to submit asterisks in place of the actual password', $this->text_domain) . "\n"; ?>
      var temp = '';
      for (var i = 0; i &lt; password.length; i++) { temp += '*'; }
      $('#user_pwd').val(temp);
      // <?php echo __('Another option is to disable the control(s) with the plain-text password(s) altogether', $this->text_domain) . "\n"; ?>
      $('#user_pwd').attr('disabled', 'true');
    }
  });
})
&lt;/script&gt;
</pre>
<p>
  <?php _e('After WordPress\' <code>init</code> hook has run, <code>$_POST[\'pwd\']</code> will contain the decrypted password.', $this->text_domain); ?>
</p>
<p>
  <?php _e('It\'s possible to pass multiple <em>passwords</em> and <em>names</em> into the <code>SemisecureLoginReimagined.encrypt</code> function. Instead of passing a single string, you can pass an array of strings. Just make sure that the password values match up with the names by keeping the elements in the same respecitve order.', $this->text_domain); ?>
</p>
<p>
  <?php _e('When creating your own integration, you might want to verify that this plugin has been activated. You can do that by using PHP\'s <code>method_exists</code> function. You can also compare the current version of this plugin by calling the <code>SemisecureLoginReimagined::version()</code> method.', $this->text_domain); ?>
</p>
<p>
  <?php _e('Two helper functions are available: <code>SemisecureLoginReimagined::is_rsa_key_ok()</code> and <code>SemisecureLoginReimagined::is_openssl_avail()</code>. These can be used to show an appropriate message to the user (both need to be true for this plugin to function properly).', $this->text_domain); ?>
</p>
<p>
  <?php _e('For best results, it\'s recommended to call <code>rng_seed_time</code> on each <em>keypress</em> or <em>mouse click</em>.', $this->text_domain); ?>
</p>
<p>
   <?php _e('For some complete examples, check out <code>inc/login_head.inc.php</code> and <code>inc/admin_head.inc.php</code> included with this plugin.', $this->text_domain); ?>
</p>

<?php elseif ($sub == 'uninstall') : ?>

<p>
	<?php _e('Deactivating this plugin does not remove any data that may have been created, such as the RSA keypair. To remove this extra data, you can uninstall this plugin here or delete it through the admin interface.', $this->text_domain); ?>
</p>
<p style="color:red;">
	<strong><?php _e('WARNING:', $this->text_domain); ?></strong><br /><?php _e('Once uninstalled, this cannot be undone. Make sure this is what you want to do. Optionally, you should backup your WordPress database first.', $this->text_domain); ?>
</p>
<p>
	<?php _e('Do you want to remove the following option(s) from the database?', $this->text_domain); ?>
</p>
<table class="widefat">
<thead>
	<tr>
		<th><?php _e('WordPress Options', $this->text_domain); ?></th>
	</tr>
</thead>
	<tr>
		<td valign="top">
			<ol>
			<li>semisecurelogin_reimagined_encrypt_admin</li>
			<li>semisecurelogin_reimagined_encrypt_login</li>
			<li>semisecurelogin_reimagined_rsa_keys</li>
			<li>semisecurelogin_reimagined_secretkey_algo</li>
			</ol>
		</td>
	</tr>
</table>
<form method="post" action="">
<p style="text-align: center;">
	<label><input type="checkbox" name="uninstall_semisecure_login_reimagined" value="yes" /> <?php _e('Yes', $this->text_domain); ?> </label><br /><br />
	<?php wp_nonce_field('uninstall_semisecure-login-reimagined'); ?>
	<input class="button" type="submit" name="Submit" value="<?php _e('UNINSTALL &raquo;', $this->text_domain); ?>" />
</p>
</form>

<?php endif; ?>
</div>