var SemisecureLoginReimagined = {
  /**
   * @param string|Array passwords (passwords and names should have the same number of elements)
   * @param string|Array names     (and be passed in the same relative order)
   * @param string nonce_js
   * @param string public_n
   * @param string public_e
   * @param string uuid
   * @param string secret_key_algo
   * @param string rand_chars
   * @param int max_rand_chars
   * @return Array|false
   */
  encrypt: function(passwords, names, nonce_js, public_n, public_e, uuid, secret_key_algo, rand_chars, max_rand_chars) {
    // if not an array, then transform the string into an array
    if (passwords.constructor != Array) {
      var temp_password = passwords;
      passwords = [];
      passwords[0] = temp_password;
    }

    if (names.constructor != Array) {
      var temp_name = names;
      names = [];
      names[0] = temp_name;
    }
    
    if (passwords.length === names.length) {
      var nonce = '';
      jQuery.ajax({
        url: nonce_js,
        cache: false,
        async: false,
        success: function(data) {
          nonce = data;
        }
      });

      var rsa = new RSAKey();
      rsa.setPublic(public_n, public_e);

      var arr = [];
      var return_arr = true;

      for (var i = 0; i < names.length; i++) {
        var secret_key = this.secret_key(rand_chars, max_rand_chars);
        var rand_pad = this.rand_pad(rand_chars); // disguise message length
        var to_be_encrypted = this.url_encode(nonce + rand_pad + passwords[i]);

        var encrypted;
        if (secret_key_algo == 'aes-cbc') {
          encrypted = Crypto.AES.encrypt(to_be_encrypted, secret_key, Crypto.mode.CBC);
        }
        else if (secret_key_algo == 'aes-ofb') {
          encrypted = Crypto.AES.encrypt(to_be_encrypted, secret_key, Crypto.mode.OFB);
        }
        else { // marc4
          encrypted = Crypto.MARC4.encrypt(to_be_encrypted, secret_key);
        }

        var res = rsa.encrypt(secret_key);

        if (res && encrypted) {
          var rsaPwd = document.createElement('input');
          rsaPwd.setAttribute('type', 'hidden');
          rsaPwd.setAttribute('name', names[i] + '__' + uuid);
          rsaPwd.value = hex2b64(res) + '|' + encrypted;

          arr[i] = rsaPwd;
        }
        else
          return_arr = false;
      }

      if (return_arr)
        return arr;
    }

    return false;
  },

  /**
   * @param string rand_chars
   * @return string
   */
  rand_pad: function(rand_chars) {
    var rand_dec = Math.floor(Math.random() * 256);
    var rand_hex = rand_dec.toString(16);
    if (rand_hex.length == 1) rand_hex = '0' + rand_hex;
    var rand_pad = rand_hex;
    for (var i = 0; i < rand_dec; i++) {
      var rand = Math.floor(Math.random() * rand_chars.length);
      rand_pad += rand_chars.substring(rand, rand+1);
    }
    return rand_pad;
  },

  /**
   * @param string rand_chars
   * @param int max_rand_chars
   * @return string
   */
  secret_key: function(rand_chars, max_rand_chars) {
    var secret_key = '';
    for (var i = 0; i < max_rand_chars; i++) {
      var rand = Math.floor(Math.random() * rand_chars.length);
      secret_key += rand_chars.substring(rand, rand+1);
    }
    return secret_key;
  },
  
  /** The following is borrowed from: http://netzreport.googlepages.com/online_tool_for_url_en_decoding.html */

  /**
   * JavaScript Tool for URL Encoding/Decoding
   * Copyright (C) 2006 Netzreport (netzreport.googlepages.com)
   *
   * Website: http://netzreport.googlepages.com/online_tool_for_url_en_decoding.html
   *
   * This program is free software; you can redistribute it and/or
   * modify it under the terms of the GNU General Public License
   * as published by the Free Software Foundation; either version 2
   * of the License, or (at your option) any later version.
   *
   * This program is distributed in the hope that it will be useful,
   * but WITHOUT ANY WARRANTY; without even the implied warranty of
   * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   * GNU General Public License for more details.
   *
   * You should have received a copy of the GNU General Public License
   * along with this program; if not, write to the Free Software
   * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
   *
   * The GNU General Public License is also available from:
   * http://www.gnu.org/copyleft/gpl.html
   *
   * A local copy of the GNU General Public License is available here:
   * http://netzreport.googlepages.com/gpl.txt
   */

  // According to RFC 3986, only characters from a set of reserved and a set
  // of unreserved characters are allowed in a URL:
  unreserved: "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz-_.~",
  //reserved: "!*'();:@&=+$,/?%#[]",
  //allowed: this.unreserved + this.reserved,
  hexchars: "0123456789ABCDEFabcdef",
  
  gethex: function(decimal) {
    return "%" + this.hexchars.charAt(decimal >> 4) + this.hexchars.charAt(decimal & 0xF);
  },
  
  url_encode: function(password) {
    var encoded = "";
    for (var i = 0; i < password.length; i++) {
      var ch = password.charAt(i);
      // Check if character is an unreserved character:
      if (this.unreserved.indexOf(ch) != -1) {
        encoded = encoded + ch;
      }
      else {
        // The position in the Unicode table tells us how many bytes are needed.
        // Note that if we talk about first, second, etc. in the following, we are
        // counting from left to right:
        //
        //   Position in   |  Bytes needed   | Binary representation
        //  Unicode table  |   for UTF-8     |       of UTF-8
        // ----------------------------------------------------------
        //     0 -     127 |    1 byte       | 0XXX.XXXX
        //   128 -    2047 |    2 bytes      | 110X.XXXX 10XX.XXXX
        //  2048 -   65535 |    3 bytes      | 1110.XXXX 10XX.XXXX 10XX.XXXX
        // 65536 - 2097151 |    4 bytes      | 1111.0XXX 10XX.XXXX 10XX.XXXX 10XX.XXXX
        
        var charcode = password.charCodeAt(i);
        // Position 0 - 127 is equal to percent-encoding with an ASCII character encoding:
        if (charcode < 128) {
          encoded = encoded + this.gethex(charcode);
        }
        // Position 128 - 2047: two bytes for UTF-8 character encoding.
        else if (charcode > 127 && charcode < 2048) {
          // First UTF byte: Mask the first five bits of charcode with binary 110X.XXXX:
          encoded = encoded + this.gethex((charcode >> 6) | 0xC0);
          // Second UTF byte: Get last six bits of charcode and mask them with binary 10XX.XXXX:
          encoded = encoded + this.gethex((charcode & 0x3F) | 0x80);
        }
        // Position 2048 - 65535: three bytes for UTF-8 character encoding.
        else if (charcode > 2047 && charcode < 65536) {
          // First UTF byte: Mask the first four bits of charcode with binary 1110.XXXX:
          encoded = encoded + this.gethex((charcode >> 12) | 0xE0);
          // Second UTF byte: Get the next six bits of charcode and mask them binary 10XX.XXXX:
          encoded = encoded + this.gethex(((charcode >> 6) & 0x3F) | 0x80);
          // Third UTF byte: Get the last six bits of charcode and mask them binary 10XX.XXXX:
          encoded = encoded + this.gethex((charcode & 0x3F) | 0x80);
        }
        // Position 65536 - : four bytes for UTF-8 character encoding.
        else if (charcode > 65535) {
          // First UTF byte: Mask the first three bits of charcode with binary 1111.0XXX:
          encoded = encoded + this.gethex((charcode >> 18) | 0xF0);
          // Second UTF byte: Get the next six bits of charcode and mask them binary 10XX.XXXX:
          encoded = encoded + this.gethex(((charcode >> 12) & 0x3F) | 0x80);
          // Third UTF byte: Get the last six bits of charcode and mask them binary 10XX.XXXX:
          encoded = encoded + this.gethex(((charcode >> 6) & 0x3F) | 0x80);
          // Fourth UTF byte: Get the last six bits of charcode and mask them binary 10XX.XXXX:
          encoded = encoded + this.gethex((charcode & 0x3F) | 0x80);
        }
      }
    }
    return encoded;
  }
}
